@@ -51,6 +51,7 @@
 #include "md5wrap.h"
 #include "metaxmlparse.h"
 #include "myhtmlparse.h"
+#include "opendocparse.h"
 #include "pkglibbindir.h"
 #include "runfilter.h"
 #include "sample.h"
@@ -62,6 +63,7 @@
 #include "utils.h"
 #include "values.h"
 #include "xmlparse.h"
+#include "xlsxparse.h"
 #include "xpsxmlparse.h"
 
 #include "gnu_getopt.h"
@@ -239,11 +241,11 @@ get_pdf_metainfo(const string & safefile, string &author, string &title,
 }
 
 static void
-generate_sample_from_csv(const string & csv_data, string & sample)
+generate_sample_from_csv(const string & csv_data, string & sample, size_t sample_size)
 {
     // Add 3 to allow for a 4 byte utf-8 sequence being appended when
-    // output is SAMPLE_SIZE - 1 bytes long.
-    sample.reserve(SAMPLE_SIZE + 3);
+    // output is sample_size - 1 bytes long.
+    sample.reserve(sample_size + 3);
     size_t last_word_end = 0;
     bool in_space = true;
     bool in_quotes = false;
@@ -287,11 +289,11 @@ generate_sample_from_csv(const string & csv_data, string & sample)
 	    in_space = false;
 	}
 
-	if (sample.size() >= SAMPLE_SIZE) {
+	if (sample.size() >= sample_size) {
 	    // Need to truncate sample.
-	    if (last_word_end <= SAMPLE_SIZE / 2) {
+	    if (last_word_end <= sample_size / 2) {
 		// Monster word!  We'll have to just split it.
-		sample.replace(SAMPLE_SIZE - 3, string::npos, "...", 3);
+		sample.replace(sample_size - 3, string::npos, "...", 3);
 	    } else {
 		sample.replace(last_word_end, string::npos, " ...", 4);
 	    }
@@ -333,7 +335,7 @@ skip_unknown_mimetype(const string & file, const string & mimetype)
 
 static void
 index_file(const string &file, const string &url, DirectoryIterator & d,
-	   map<string, string>& mime_map)
+	   map<string, string>& mime_map, size_t sample_size)
 {
     string ext;
     const char * dot_ptr = strrchr(d.leafname(), '.');
@@ -554,11 +556,11 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	{
 	    // Inspired by http://mjr.towers.org.uk/comp/sxw2text
 	    string safefile = shell_protect(file);
-	    string cmd = "unzip -p " + safefile + " content.xml styles.xml";
+	    string cmd = "unzip -p " + safefile + " content.xml ; unzip -p " + safefile + " styles.xml";
 	    try {
-		XmlParser xmlparser;
-		xmlparser.parse_html(stdout_to_string(cmd));
-		dump = xmlparser.dump;
+		OpenDocParser parser;
+		parser.parse_html(stdout_to_string(cmd));
+		dump = parser.dump;
 	    } catch (ReadError) {
 		skip_cmd_failed(file, cmd);
 		return;
@@ -585,6 +587,7 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	    }
 	} else if (startswith(mimetype, "application/vnd.openxmlformats-officedocument.")) {
 	    const char * args = NULL;
+	    string safefile = shell_protect(file);
 	    string tail(mimetype, 46);
 	    if (startswith(tail, "wordprocessingml.")) {
 		// unzip returns exit code 11 if a file to extract wasn't found
@@ -592,7 +595,18 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 		// no footers.
 		args = " word/document.xml word/header\\*.xml word/footer\\*.xml 2>/dev/null||test $? = 11";
 	    } else if (startswith(tail, "spreadsheetml.")) {
-		args = " xl/sharedStrings.xml";
+		// Extract the shared string table first, so our parser can
+		// grab those ready for parsing the sheets which will reference
+		// the shared strings.
+		string cmd = "unzip -p " + safefile + " xl/sharedStrings.xml ; unzip -p " + safefile + " xl/worksheets/sheet\\*.xml";
+		try {
+		    XlsxParser parser;
+		    parser.parse_html(stdout_to_string(cmd));
+		    dump = parser.dump;
+		} catch (ReadError) {
+		    skip_cmd_failed(file, cmd);
+		    return;
+		}
 	    } else if (startswith(tail, "presentationml.")) {
 		// unzip returns exit code 11 if a file to extract wasn't found
 		// which we want to ignore, because there may be no notesSlides
@@ -603,18 +617,20 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 		skip_unknown_mimetype(file, mimetype);
 		return;
 	    }
-	    string safefile = shell_protect(file);
-	    string cmd = "unzip -p " + safefile + args;
-	    try {
-		XmlParser xmlparser;
-		xmlparser.parse_html(stdout_to_string(cmd));
-		dump = xmlparser.dump;
-	    } catch (ReadError) {
-		skip_cmd_failed(file, cmd);
-		return;
+
+	    if (args) {
+		string cmd = "unzip -p " + safefile + args;
+		try {
+		    XmlParser xmlparser;
+		    xmlparser.parse_html(stdout_to_string(cmd));
+		    dump = xmlparser.dump;
+		} catch (ReadError) {
+		    skip_cmd_failed(file, cmd);
+		    return;
+		}
 	    }
 
-	    cmd = "unzip -p " + safefile + " docProps/core.xml";
+	    string cmd = "unzip -p " + safefile + " docProps/core.xml";
 	    try {
 		MetaXmlParser metaxmlparser;
 		metaxmlparser.parse_html(stdout_to_string(cmd));
@@ -727,7 +743,7 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 		// FIXME: What charset is the file?  Look at contents?
 	    }
 
-	    generate_sample_from_csv(dump, sample);
+	    generate_sample_from_csv(dump, sample, sample_size);
 	} else if (mimetype == "application/vnd.ms-outlook") {
 	    string cmd = get_pkglibbindir() + "/outlookmsg2html " + shell_protect(file);
 	    MyHtmlParser p;
@@ -796,6 +812,16 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	    return;
 	}
 
+	// Remove any trailing formfeeds, so we don't consider them when
+	// considering if we extracted any text (e.g. pdftotext outputs a
+	// formfeed between each page, even for blank pages).
+	//
+	// If dump contain only formfeeds, then trim_end will be string::npos
+	// and ++trim_end will be 0, which is the correct new size.
+	string::size_type trim_end = dump.find_last_not_of('\f');
+	if (++trim_end != dump.size())
+	    dump.resize(trim_end);
+
 	if (dump.empty()) {
 	    switch (empty_body) {
 		case EMPTY_BODY_INDEX:
@@ -812,9 +838,9 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 
 	// Produce a sample
 	if (sample.empty()) {
-	    sample = generate_sample(dump, SAMPLE_SIZE);
+	    sample = generate_sample(dump, sample_size);
 	} else {
-	    sample = generate_sample(sample, SAMPLE_SIZE);
+	    sample = generate_sample(sample, sample_size);
 	}
 
 	// Put the data in the document
@@ -971,7 +997,7 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 
 static void
 index_directory(const string &path, const string &url_, size_t depth_limit,
-		map<string, string>& mime_map)
+		map<string, string>& mime_map, size_t sample_size)
 {
     if (verbose)
 	cout << "[Entering directory \"" << path.substr(root.size()) << "\"]"
@@ -996,11 +1022,11 @@ index_directory(const string &path, const string &url_, size_t depth_limit,
 			}
 			url += '/';
 			file += '/';
-			index_directory(file, url, new_limit, mime_map);
+			index_directory(file, url, new_limit, mime_map, sample_size);
 			break;
 		    }
 		    case DirectoryIterator::REGULAR_FILE:
-			index_file(file, url, d, mime_map);
+			index_file(file, url, d, mime_map, sample_size);
 			break;
 		    default:
 			skip(file, "Not a regular file",
@@ -1016,6 +1042,35 @@ index_directory(const string &path, const string &url_, size_t depth_limit,
     }
 }
 
+static off_t
+parse_size(char* p)
+{
+    // Don't want negative numbers, infinity, NaN, or hex numbers.
+    if (C_isdigit(p[0]) && (p[1] | 32) != 'x') {
+	double arg = strtod(p, &p);
+	switch (*p) {
+	    case '\0':
+		break;
+	    case 'k': case 'K':
+		arg *= 1024;
+		++p;
+		break;
+	    case 'm': case 'M':
+		arg *= (1024 * 1024);
+		++p;
+		break;
+	    case 'g': case 'G':
+		arg *= (1024 * 1024 * 1024);
+		++p;
+		break;
+	}
+	if (*p == '\0') {
+	    return off_t(arg);
+	}
+    }
+    return -1;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1026,6 +1081,7 @@ main(int argc, char **argv)
     bool delete_removed_documents = true;
     string baseurl;
     size_t depth_limit = 0;
+    size_t sample_size = SAMPLE_SIZE;
 
     static const struct option longopts[] = {
 	{ "help",	no_argument,		NULL, 'h' },
@@ -1046,6 +1102,7 @@ main(int argc, char **argv)
 	{ "verbose",	no_argument,		NULL, 'v' },
 	{ "empty-docs",	required_argument,	NULL, 'e' },
 	{ "max-size",	required_argument,	NULL, 'm' },
+	{ "sample-size",required_argument,	NULL, 'E' },
 	{ 0, 0, NULL, 0 }
     };
 
@@ -1225,7 +1282,7 @@ main(int argc, char **argv)
 
     string dbpath;
     int getopt_ret;
-    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:F:l:s:pfSVe:im:",
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:F:l:s:pfSVe:im:E:",
 					 longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
@@ -1257,6 +1314,8 @@ main(int argc, char **argv)
 "  -S, --spelling           index data for spelling correction\n"
 "  -m, --max-size           maximum size of file to index (in bytes or with a\n"
 "                           suffix of 'K'/'k', 'M'/'m', 'G'/'g')\n"
+"  -E, --sample-size=SIZE   sets the maximum number of bytes for the document\n"
+"                           text sample. (default SIZE = 512)\n"
 "  -v, --verbose            show more information about what is happening\n"
 "      --overwrite          create the database anew (the default is to update\n"
 "                           if the database already exists)" << endl;
@@ -1361,31 +1420,20 @@ main(int argc, char **argv)
 	case 'v':
 	    verbose = true;
 	    break;
+	case 'E': {
+	    off_t arg = parse_size(optarg);
+	    if (arg >= 0) {
+		sample_size = size_t(arg);
+		break;
+	    }
+	    cerr << PROG_NAME": bad sample size '" << optarg << "'" << endl;
+	    return 1;
+	}
 	case 'm': {
-	    // Don't want negative numbers, infinity, NaN, or hex numbers.
-	    char * p = optarg;
-	    if (C_isdigit(p[0]) && (p[1] | 32) != 'x') {
-		double arg = strtod(p, &p);
-		switch (*p) {
-		    case '\0':
-			break;
-		    case 'k': case 'K':
-			arg *= 1024;
-			++p;
-			break;
-		    case 'm': case 'M':
-			arg *= (1024 * 1024);
-			++p;
-			break;
-		    case 'g': case 'G':
-			arg *= (1024 * 1024 * 1024);
-			++p;
-			break;
-		}
-		if (*p == '\0') {
-		    max_size = off_t(arg);
-		    break;
-		}
+	    off_t size = parse_size(optarg);
+	    if (size >= 0) {
+		max_size = size;
+		break;
 	    }
 	    cerr << PROG_NAME": bad max size '" << optarg << "'" << endl;
 	    return 1;
@@ -1496,7 +1544,7 @@ main(int argc, char **argv)
 	}
 	indexer.set_stemmer(stemmer);
 
-	index_directory(root + start_url, baseurl + start_url, depth_limit, mime_map);
+	index_directory(root + start_url, baseurl + start_url, depth_limit, mime_map, sample_size);
 	if (delete_removed_documents && old_docs_not_seen) {
 	    if (verbose) {
 		cout << "Deleting " << old_docs_not_seen << " old documents which weren't found" << endl;
